/*
**--------------------------------------------------------------------
** VENTANA.C
**
** Funciones para Scroll de una ventana por pantalla
**
** Jos Manuel Mulledes Roncero. Mayo, 1988
**--------------------------------------------------------------------
*/

#include <stdio.h>
#include <conio.h>
#include <stdio.h>
#include <alloc.h>
#include <process.h>  /* Necesario para el exit */
#include <dos.h>      /* Necesario para el delay */

/*
** Esta funcin realiza el movimiento de una ventana de la pantalla de
** texto a una nueva posicion. Se utiliza acceso directo a la memoria de
** video. La pantalla tiene que estar en modo texto color 80 columnas.
** Direccion 0xB800:0000.
** La estructura de la ventana ser de la siguiente manera.
*/

typedef struct {
	int OrigenX ; 				/* Coordenada X del origen              */
	int OrigenY ;				/* Coordenada Y del Origen              */
	int FinX ;			  	/* Coordenada X del Fin                 */
	int FinY;				/* Coordenada Y del Fin                 */
	unsigned char *PantallaAnterior ;  	/* Donde se guarda la pantalla anterior.*/
} StrVentana ;

/* Prototipos de funciones.*/
int MueveVentana ( int , int , StrVentana * ) ;
int CreaVentana ( int , int , int , int , StrVentana * ) ;



void main ( void )
{
	StrVentana Ventana1 ;
	register int x ;
	int retraso=0;
	int puntos[] = { 7,10,9, 8,11, 8,13, 8,15,10,15,12,
	  15,14,13,16,11,16,9,16,7,14,7,12} ;

	clrscr ();
	printf ( "Introduce un retardo:");
	scanf ( "%d" , &retraso ) ;
	for ( x = 1 ; x < 100 ; x++ ) printf ("--------------------");

	CreaVentana( 1 , 1, 10 , 20 , &Ventana1 ) ;

	getch() ;

	x = 0 ;
	do {
		MueveVentana ( puntos[x++] , puntos[x++] , &Ventana1 ) ;
		if ( x > 24 ) x = 0 ;
		delay ( retraso ) ;

	}while( !kbhit()) ;

}

/*
** Funcion CreaVentana: Crea una ventana en las coordenadas indicadas. La
** ventana se le pasa por referencia.
*/

int CreaVentana ( int OrX , int OrY , int FX , int FY , StrVentana *Auxiliar )
{
	char far *video = (char far*) 0xB8000000 ;
	char far *t ;
	unsigned char *copia ;
	register int i , j ;

	Auxiliar->OrigenX = OrX ;
	Auxiliar->OrigenY = OrY ;
	Auxiliar->FinX    = FX ;
	Auxiliar->FinY    = FY ;

	/* Salvar la pantalla anterior ;*/
	Auxiliar->PantallaAnterior = (unsigned int *)malloc (
		(Auxiliar->FinX-Auxiliar->OrigenX)*
		(Auxiliar->FinY-Auxiliar->OrigenY) * 2) ;

	/* Guardar la direccion de comienzo de Auxiliar.PantallaAnterior.*/
	copia = Auxiliar->PantallaAnterior ;

	for( i = OrY ; i < FY ; i++ )
	{
		for ( j = OrX ; j < FX ; j++ )
		{
			t = video +(j*160) +i*2 ;		/* Calcular la direccion */
			*Auxiliar->PantallaAnterior++ = *t++ ;  /* Leer el caracter      */
			*Auxiliar->PantallaAnterior++ = *t ;	/* Leer el atributo      */
			*(t-1) = '' ;
		}
	}

	Auxiliar->PantallaAnterior = copia ;

	return 1 ;
}


/*
** MueveVentana: Mueve la ventana pasada por referencia a una nueva posicion
*/
int MueveVentana ( int NuevaX, int NuevaY, StrVentana *Ventana )
{
	char far *video = (char far *)0xB8000000 ;
	char far *t ;
	register int i , j ;
	int DiffX , DiffY ;
	unsigned char *Actual , *copia , *copia2 ;

	/* Reservar memoria para guardar la imagen actual de la ventana */
	Actual = (unsigned char *)malloc (
		(Ventana->FinX-Ventana->OrigenX)*
		(Ventana->FinY-Ventana->OrigenY)*2) ;

	/* Guardar la direccion de comienzo de Actual */
	copia = Actual ;

	/* Salva la imagen de la ventana en su estado actual */

	for( i = Ventana->OrigenY ; i < Ventana->FinY ; i++ )
	{
		for ( j = Ventana->OrigenX ; j < Ventana->FinX ; j++ )
		{
			t = video +(j*160) +i*2 ; 	/* Calcular la direccion */
			*Actual++=  *t++ ;  		/* Salvar el caracter    */
			*Actual++ = *t   ;  		/* Poner el atributo     */
		}
	}

	/* Restaurar la direccion de inicio de Actual */
	Actual = copia ;

	/* Guardo la direccion de inicio de Ventana->PantallaAnterior */
	copia2 = Ventana->PantallaAnterior ;

	/* Restauro la pantalla anterior */
	for( i = Ventana->OrigenY ; i < Ventana->FinY ; i++ )
	{
		for ( j = Ventana->OrigenX ; j < Ventana->FinX ; j++ )
		{
			t = video +(j*160) +i*2 ; 		/* Calcular la direccion */
			*t++= *Ventana->PantallaAnterior++ ;  	/* Poner el caracter     */
			*t  = *Ventana->PantallaAnterior++ ;  	/* Poner el atributo     */
		}
	}

	/* Restauro la direccion de inicio de Ventana->PantallaAnterior */
	Ventana->PantallaAnterior = copia2 ;

	/* Calculos */
	DiffX = Ventana->FinX - Ventana->OrigenX;
	DiffY = Ventana->FinY - Ventana->OrigenY;

	Ventana->OrigenX = NuevaX ;
	Ventana->FinX    = NuevaX+DiffX ;
	Ventana->OrigenY = NuevaY ;
	Ventana->FinY    = NuevaY+DiffY ;

	/* Salvo la pantalla anterior de la nueva posicion en Ventana->PantallaAnterior */
	for( i = Ventana->OrigenY ; i < Ventana->FinY ; i++ )
	{
		for ( j = Ventana->OrigenX ; j < Ventana->FinX ; j++ )
		{
			t = video +(j*160) +i*2 ; 		/* Calcular la direccion */
			*Ventana->PantallaAnterior++=  *t++ ;  	/* Salvar el caracter	 */
			*Ventana->PantallaAnterior++ = *t   ;  	/* Poner el atributo     */
		}
	}

	/* Restauro la direccion de inicio de Ventana->PantallaAnterior */
	Ventana->PantallaAnterior = copia2 ;

	/* Pongo la ventana en la nueva posicion */
	for( i = Ventana->OrigenY ; i < Ventana->FinY ; i++ )
	{
		for ( j = Ventana->OrigenX ; j < Ventana->FinX ; j++ )
		{
			t = video +(j*160) +i*2 ; 	/* Calcular la direccion */
			*t++= *Actual++ ;  		/* Pongo el caracter     */
			*t  = *Actual++ ;  		/* Pongo el atributo     */
		}
	}

	free ( Actual ) ;

	return 1 ;
}
